def get_metrics(response):
    """
    Extract asked metrics from api response
    @list_metrics : list of dict
    """
    list_metrics = []
    for i in response['reports'][0]['columnHeader']['metricHeader']['metricHeaderEntries']:
        list_metrics.append(i['name'])
    return list_metrics


def get_dimensions(response):
    """
      Extract asked dimensions from api response
      @list_dimensions : list of dict
    """
    return response['reports'][0]['columnHeader']['dimensions']


def extract_api_data(response):
    """
    Extract all data from api response
    """
    types = {}
    for key in response['reports'][0]["columnHeader"]["metricHeader"]["metricHeaderEntries"]:
        types[key["name"]] = key["type"]

    try:
        rows = response['reports'][0]['data']['rows']
    except:
        return []
    sampling = False
    samples_read_counts = response['reports'][0]['data'].get('samplesReadCounts')
    sampling_space_sizes = response['reports'][0]['data'].get('samplingSpaceSizes')
    if samples_read_counts:
        print("SAMPLING")
        sampling = True

    metric_response = get_metrics(response)
    dimensions_response = get_dimensions(response)
    data = []
    for row in rows:
        d = {}
        j = 0
        for i in dimensions_response:
            d[i] = row['dimensions'][j]
            j = j + 1
        j = 0
        for i in metric_response:
            d[i] = row['metrics'][0]['values'][j]
            j = j + 1
        d["sampling"] = sampling

        data.append(d)
    return data, types



import copy

from dbstream import DBStream

from pyganalytics.core.mapping_types import mapping_types


def send_to_db(dbstream: DBStream, data, all_batch_id):
    all_batch_id = ["'" + e + "'" for e in all_batch_id]
    table_name = data["table_name"]
    if all_batch_id:
        try:
            query = "DELETE FROM %s WHERE batch_id IN (%s);" % (table_name, ",".join(all_batch_id))
            dbstream.execute_query(query)
        except:
            print("Table does not exist")
    data["columns_name"] = [r.replace(":", "_").lower() for r in data["columns_name"]]
    types = {}
    for t in data["types"]:
        types[t.replace(":", "_").lower()] = mapping_types[data["types"][t]]
    data["types"] = types
    copy_data = copy.deepcopy(data)
    total_length = len(copy.deepcopy(data)["rows"])
    try:
        dbstream.send_data(data, replace=False)
    except:
        i = 0
        while i < total_length:
            small_rows = copy_data["rows"][i:i + 50]
            print(small_rows)
            dbstream.send_data({
                "table_name": copy_data["table_name"],
                "columns_name": copy_data["columns_name"],
                "rows": small_rows,
                "types": copy_data["types"]
            }, replace=False)
            i = i + 50


def def_table_name(prefix_schema, report_name, time_increment, googleanalytics):
    table_name = report_name + "_" + time_increment.replace(":", "_")
    if googleanalytics.full_schema_name:
        return googleanalytics.full_schema_name + "." + table_name
    output_storage_name = "ga." + table_name
    if prefix_schema:
        output_storage_name = prefix_schema + "_" + output_storage_name
    return output_storage_name



import datetime

from dbstream import DBStream

from pyganalytics.core.extract.manage_view import get_all_access_view


def get_start_end(start, end, table_name, dbstream: DBStream):
    if not end:
        end = str(datetime.datetime.now())[:10]
    if not start:
        schema_name = table_name.split(".")[0]
        table_name = table_name.split(".")[1]
        max_date = dbstream.get_max(schema=schema_name, table=table_name, field="date", filter_clause="")
        if not max_date:
            start = str(datetime.datetime.strptime(end, "%Y-%m-%d") + datetime.timedelta(days=-10))[:10]
        else:
            if isinstance(max_date, str):
                max_date = datetime.datetime.strptime(max_date, "%Y-%m-%d")
            start = str(
                max_date +
                datetime.timedelta(days=-1)
            )[:10]
    return start, end


def get_all_view_id(googleanalytics, all_view_id):
    if all_view_id == "*":
        all_view_id = get_all_access_view(googleanalytics)
        all_view_id = [ai["view_id"] for ai in all_view_id]
    elif not all_view_id:
        print("No list of view_id given as argument")
        exit()
    return all_view_id
